/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             DeltaLog management
*  SCOPE :              DeltaLog management
*  FILE :               delta.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __delta_HXX
#define __delta_HXX

/* IMPORTS *******************************************************************/

#ifndef __class_HXX
#include "class.hxx"
#endif

#ifndef __oid_HXX
#include "oid.hxx"
#endif

#ifndef __log_HXX
#include "log.hxx"
#endif

#ifndef __pagehdl_HXX
#include "pagehdl.hxx"
#endif

#ifndef __yootypes_H
#include "yootypes.h"
#endif

#ifndef _linkedlist_HXX
#include "linkedlist.hxx"
#endif

/*****************************************************************************/

class Delta : public LinkedElem {

  protected :
  
  static LinkedList _deltalist ;
  
  public :

  static DeltaVue* createVue() ;     // Create a new deltavue for a vue
  static DeltaLog* createLog(char*); // Create a new deltalog for a commit
  virtual int read(Page*, PID) ;     // Read page design by pid into p  
  virtual int delta(PID) ;           // Return TRUE if the page is in deltalog
  virtual int islog() = 0 ;          // Return TRUE if it is a deltalog
  virtual int isactive() = 0 ;       // Return TRUE if it is a deltalog in use
} ;


class DeltaVue : public Delta {

  public :

  DeltaVue() {} ;
  ~DeltaVue() ;
  int islog() ;              // Return FALSE
  int isactive() ;           // Return FALSE
} ;


class DeltaLog : public Delta, public Log {

  class PageHdlLog : public PageHdl {
    private :

    int _numpage ;
    
    public :

    PageHdlLog(int np, PID&) ;
    int numpage() ;
  } ;
  
  private :

  PageSet _logindex ;  
  
  public :

  DeltaLog(char* name) : Log(name) {} ;
  ~DeltaLog() ;
  int read(Page*, PID) ;    // Read page referenced by pid into p  
  int write(Page*, PID) ;   // Write page referenced by pid from p
  int delta(PID) ;          // Return TRUE if the page is in deltalog
  void close() ;            // Close a deltalog. Stay in the list for delta use
  int islog() ;             // Return TRUE
  int isactive() ;          // Return TRUE if it is in use
} ;


inline DeltaLog::PageHdlLog::PageHdlLog(int np, PID& pid) :
PageHdl(pid), _numpage(np) {}

inline int DeltaLog::PageHdlLog::numpage() {
  return _numpage ;
}

#endif
  
